/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const truncateNum = 60;
var lockKey = -1;

function init(){
	loadErrorPrefs();
	onDuplicateLoad();
	startLookup();
	setBuildType($("radiogroupBT"));
	$("intervalSlider").value = Prefs.getInt("indexDelay");
	var path = Prefs.getFile("indexPath").path;
	$("pathTooltip").firstChild.value = path;
	$("pathInfo").value = path.truncate(truncateNum, ' [..]');
	$("loadFolder").disabled = Prefs.getBool("indexPathDefault");
	
	if(Prefs.getBool('isRunning')){
		$("intervalSlider").disabled = true;
		$("loadFolder").disabled = true;
		$("loadFolder_check").disabled = true;
		
		$("buildButton").disabled = true;
		$("deleteButton").disabled = true;
		
		$("CHECK").disabled = true;
		$("DISCARD").disabled = true;
		$("DUPLICATES").disabled = true;
	}
}

function uninit(){
	Prefs.setInt("indexDelay", $("intervalSlider").value);
}

/*
	Sets the current choosed build type and resets
	preferences if the autimatic mode was selected.

	@param item radiogroup to get the selected item
*/
function setBuildTypePrefs(item){
	if(item.selectedItem.id == 'AUTO')
		Prefs.setBool('firstRun', true);

}

function setBuildType(item){
	if(!Prefs.getBool('isRunning')){
		var intervalSlider = $("intervalSlider");
		var buildButton = $("buildButton");
		var deleteButton = $("deleteButton");
		
		switch(item.selectedItem.id){
			case 'MANU': 
				intervalSlider.disabled = true;
				buildButton.disabled = false;
				deleteButton.disabled = false;
				break;
			case 'AUTO': 
				intervalSlider.disabled = false; 
				buildButton.disabled = true; 
				deleteButton.disabled = true; 
				break;
			default: break;
		}
	}
}

function setLocationValue(slider){
	var elem = $("info");
	var val = elem.value.split(" ")[1];
	elem.value = slider.value + " " + val;
}

function setUserDefinedPath(checkbox){
	if(!checkbox.checked){
		$("loadFolder").disabled = true;
		Prefs.remove("alreadyAddedBookmarks");
	}else {
		$("loadFolder").disabled = false;
	}
}

function setIndexPath(){
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
	fp.init(window, "Select folder", nsIFilePicker.modeGetFolder);
	var res = fp.show();
	if (res == nsIFilePicker.returnOK){
		var path = fp.file.path;
		Prefs.setFile("indexPath", fp.file);
		$("pathTooltip").firstChild.value = path;
		$("pathInfo").value = path.truncate(truncateNum, ' [..]');
		Prefs.remove("alreadyAddedBookmarks");
	}
}

function onDuplicateLoad(){
	var button = $("DUPLICATES");
	var l = button.label;
	l = l.replace(/\s(.*)/gi, String.empty);
	var duplicates = Prefs.getBranch('dupBm');
	button.label = l + " (" + duplicates.length + ")";
	button.disabled = duplicates.length == 0;
}

/*
	Shows currently indexed bookmarks.
	
	Note:
		Never change the user listing (indexed bookmarks) while lookup, 
		because the delete function recieves the tree indexes that are
		used to refer to the corresponding index document.
*/
function startLookup(){
	try {
		var array = Index._lookup();
		var e = array.enumerate();

		var indexed = [];
		
		var resources = Book.getResourceHash();
		var errors = Prefs.getBranch('invBm').toHash();

		var tree_indexed = $("INDEXED");
		tree_indexed.clear();
		
		var untraceableBookmarks = [];
		var pos = 0;
		
		while (e.hasMoreElements()) {
			var lookupNode = e.getNext().QueryInterface(Components.interfaces.ILookupNode);

			var status = 0;
			if(!resources.contains(lookupNode.primaryKey)){
				status = 2;
				untraceableBookmarks.push(pos);
			}else if(errors.contains(lookupNode.primaryKey)){
				status = 1;
			}
			
			tree_indexed.add(lookupNode.name, lookupNode.primaryKey, status);
			pos++;
			
			indexed[lookupNode.primaryKey] = 1;
		}
		
		//checks preferences if cleanup flag is set
		//and removes all untraceable bookmarks
		doCleanup(untraceableBookmarks);
		
		$("INDEXED_CTR").value = "(" + tree_indexed.size + "), " + formatBytes(getDirectoryInfo(getSearchDir()));
	} catch(e){
		alert(e);
	}
}

function doCleanup(untraceableBookmarks){
	if(Prefs.getBool('cleanupUntraceableBookmarks')){
		Index._delete(untraceableBookmarks);
	}
}

function doCheck(){

	//clear prefs
	Prefs.getBranch('invBm').clear();
	Prefs.getBranch('dupBm').clear();
	
	//clear last log files
	clearLogs();
	
	//clear tree and button information
	$("ERRORTREE").clear();
	$("ERROR_COUNT").value = "(0)";
	
	$("OTHERTREE").clear();
	$("OTHER_COUNT").value = "(0)";
	
	onDuplicateLoad();

	//init crawler
	Crawler.init();
	Crawler.setBookmarks(Book.getResourceHash());

	//init filter
	Filter.init();
	Filter.setCrawler(Crawler);
	Filter.setCheck(true);
	Filter.setView(View);
	
	var retVals = {filter : Filter};
	openDialog("chrome://bookmarktools/content/popup.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);

	//alert("To keep the index synchronized you can start this process only with one browser instance. So please close all other browsers and try again.");

}

function doDuplicates(){
	var retVals = {size : "empty"};
	openDialog("chrome://bookmarktools/content/filter.xul", "_blank", "chrome, centerscreen, resizable=yes, modal", retVals);
	
	var size = retVals["size"];
	if(size != "empty"){
		var button = $("DUPLICATES");
		var l = button.label;
		l = l.replace(/\s(.*)/gi, String.empty);
		
		button.label = l + " (" + size + ")";
		button.disabled = size == 0;
	}
}

function doAdd(){

	//clear prefs
	Prefs.getBranch('invBm').clear();
	Prefs.getBranch('dupBm').clear();
	
	//clear last log files
	clearLogs();
	
	//clear tree and button information
	$("ERRORTREE").clear();
	$("ERROR_COUNT").value = "(0)";
	
	$("INDEXED").clear();
	$("INDEXED_CTR").value = "(0)";
	
	$("OTHERTREE").clear();
	$("OTHER_COUNT").value = "(0)";
	
	onDuplicateLoad();
	
	//clear search dir
	clear(getSearchDir());
	
	var bookmarks = Book.getResourceHash();

	//init crawler
	Crawler.init();
	Crawler.setBookmarks(bookmarks);

	//init filter
	Filter.init();
	
	//bind with index
	Index.setMode(MANUAL);
	Index.setFilter(Filter);
	
	Filter.setCrawler(Crawler);
	Filter.setView(View);
	
	var retVals = {index : Index};
	openDialog("chrome://bookmarktools/content/popup.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);

}

function tryBuildAutomatic(){

	//checks if the build type 'AUTOMATIC' is set and 
	//if the current time is greater then the last build time + delay
	if(Index.isValidBuildTime()){
		
		if(lockKey != -1){
		
			//alert("Prefs.getBool('firstRun') = " + Prefs.getBool('firstRun'));
		
			if(Prefs.getBool('firstRun')){ //first run?
				//clear prefs
				Prefs.getBranch('invBm').clear();
				Prefs.getBranch('dupBm').clear();
				
				//clear last log files
				clearLogs();
				
				//clear build directory
				clear(getBuildDir());
				
				//alert("Build dir successfully cleared!");
				
				//generate log file
				var file = generateFile(getBuildDir(), "alreadyAddedBookmarks.log");
				Prefs.setFile("alreadyAddedBookmarks", file);
				
				//alert("File generated and flag reset!");
			}
		
			//init crawler
			Crawler.init();

			//init filter
			Filter.init();
			var indexState = Filter.getBookmarksFromFile();
			Crawler.setBookmarks(indexState.remainingBookmarks);
			
			//alert("File reading..");
			//alert("indexState.remainingBookmarks.length = " + indexState.remainingBookmarks.length());
			
			//bind with index
			Index.setMode(AUTOMATIC);
			Index.setFilter(Filter);
			
			Filter.setCrawler(Crawler, indexState.currentIndexed, indexState.allBookmarks);
			
			//alert("indexState.currentIndexed = " + indexState.currentIndexed);
			//alert("indexState.allBookmarks = " + indexState.allBookmarks);
			
			BrowserView.init();
			//alert("BrowserView.init()");
			
			Filter.setView(BrowserView);
			
			Index._build();
		
			//sets first run to false
			Prefs.setBool('firstRun', false);
			Prefs.setBool('isRunning', true);
		}
	}
	
}

function doDelete(){
	var tree = $('INDEXED');
	var nodes = tree.selection;
	tree.unselect;

	//set path to search directory
	Index.setMode(MANUAL);
	Index._delete(nodes);

	startLookup();
}
